/** Master's Thesis code
 * <p>
 * Copyright (c) 2005 Eric Eaton
 * <p>
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * <p>
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * <p>
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses/.
 * 
 * @author Eric Eaton (EricEaton@umbc.edu) <br>
 *         University of Maryland Baltimore County
 * 
 * @version 1.0
 *
 */
public class Constraint {
	private int m_firstIdx;
	private int m_secondIdx;
	private ConstraintType m_constraintType;
	private double m_weight;
	
	public enum ConstraintType {MUST_LINK, CANNOT_LINK, DONT_CARE};

	
	/** Constructor with a default constraint weight of 1.0 and type of DONT_CARE
	 * @param firstIdx
	 * @param secondIdx
	 */
	public Constraint(int firstIdx, int secondIdx) {
		this(firstIdx, secondIdx, ConstraintType.DONT_CARE, 1.0);
	}
	
	
	/** Constructor with a default constraint weight of 1.0
	 * @param firstIdx
	 * @param secondIdx
	 * @param constraintType
	 */
	public Constraint(int firstIdx, int secondIdx, ConstraintType constraintType) {
		this(firstIdx, secondIdx, constraintType, 1.0);
	}
	
	
	/** Constructor
	 * @param firstIdx
	 * @param secondIdx
	 * @param constraintType
	 * @param weight
	 */
	public Constraint(int firstIdx, int secondIdx, ConstraintType constraintType, double weight) {
		m_firstIdx =  (firstIdx < secondIdx) ? firstIdx  : secondIdx;
		m_secondIdx = (firstIdx < secondIdx) ? secondIdx : firstIdx;
		m_constraintType = constraintType;
		m_weight = weight;
	}
	
	
	public int getFirstIdx() {
		return m_firstIdx;
	}

	public int getSecondIdx() {
		return m_secondIdx;
	}

	public ConstraintType getConstraintType() {
		return m_constraintType;
	}

	public double getWeight() {
		return m_weight;
	}


	public void setWeight(double weight) {
		this.m_weight = weight;
	}
	
	
	public boolean equals (Object obj) {
	    if (obj instanceof Constraint) {
	      Constraint c = (Constraint) obj;
	      return (this.m_firstIdx  == c.m_firstIdx  &&
		          this.m_secondIdx == c.m_secondIdx && 
		          this.m_constraintType == c.m_constraintType);
	    }
	    return super.equals(obj);
	}
	
	public String toString() {
		return this.getIdxDescription() + "," + m_constraintType + "," + m_weight;
	}
	
	public int hashCode() {
		return toString().hashCode();
	}
	
	
	public String getIdxDescription() {
		return m_firstIdx + "," + m_secondIdx;
	}
}
